@@ -1,16 +1,13 @@
 from contextlib import contextmanager, nullcontext
-from dataclasses import dataclass, field
 from functools import partial
 import logging
 from importlib import import_module
 import inspect
-from io import UnsupportedOperation
 from mock import patch
 from multiprocessing import Process, get_context
 import os
 from random import randrange
 import requests
-from shutil import rmtree
 import signal
 import sys
 from threading import Thread
@@ -18,6 +15,8 @@ from typing import List, Optional
 import time
 import uuid
 
+
+from asgi_correlation_id import correlation_id
 import fastapi as fa
 from prometheus_client import multiprocess, CollectorRegistry
 from prometheus_fastapi_instrumentator import Instrumentator
@@ -25,6 +24,7 @@ from pydantic import BaseModel, Extra, Field
 import uvicorn
 
 from chaiverse.logging import get_service_logger_config
+from chaiverse.fastapi_instrumentator import FastAPIInstrumentator
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
@@ -94,12 +94,14 @@ class ChaiverseServerConfig(BaseModel):
     limit_max_requests: int = -1
     log_level: str = "info"
     loki_base_url: str = ""
+    prometheus_labels: Optional[dict] = {}
     # Specify the buckets to use when exporting route latencies to
     # prometheus histogram. This is a configurable option as we want
     # to avoid publishing too many time series to prometheus if we
     # don't need them
     prometheus_latency_buckets: List[float] = [0.1, 0.5, 1]
     prometheus_path: Optional[str] = None
+    trace_header_name: Optional[str] = 'traceparent'
 
     class Config:
         extra = Extra.allow
@@ -124,6 +126,7 @@ class ChaiverseServer:
     def __init__(self, fastapi_app: fa.FastAPI, config):
         self.config = config
         self.fastapi_app_factory = partial(self._prepare_fastapi, fastapi_app)
+        self.prometheus_instrumentator = None
 
     @contextmanager
     def serve(self):
@@ -182,12 +185,9 @@ class ChaiverseServer:
         if isinstance(fastapi_app, str):
             fastapi_app = _resolve_fastapi_app(fastapi_app)
         _install_general_exception_handler(fastapi_app)
-        _install_prometheus_instrumentator(
-            fastapi_app,
-            self.config.name,
-            self.config.prometheus_latency_buckets,
-            self.config.prometheus_path
-        )
+        self.prometheus_instrumentator = FastAPIInstrumentator(fastapi_app, self.config)
+        self.prometheus_instrumentator.instrument()
+
         fastapi_app.state.chaiverse_server_config = self.config
         return fastapi_app
 
@@ -209,55 +209,22 @@ class ChaiverseServer:
 def _install_general_exception_handler(fastapi_app):
     @fastapi_app.exception_handler(Exception)
     def runtime_exception_handler(request: fa.Request, exc: Exception):
-        return exception_response(500, exc)
+        return exception_response(fastapi_app, 500, exc)
 
 
-def _install_prometheus_instrumentator(fastapi_app, name, latency_buckets=[0.1, 0.5, 1], path=None):
-    if _should_install_prometheus_middleware(fastapi_app):
-        prometheus_dir = path if path else f"/tmp/prometheus/{uuid.uuid4()}"
-        os.environ["PROMETHEUS_MULTIPROC_DIR"] = prometheus_dir
-        if not os.path.exists(prometheus_dir):
-            os.mkdir(prometheus_dir)
-
-        registry = CollectorRegistry()
-        multiprocess.MultiProcessCollector(registry)
-
-        instrumentator = Instrumentator()
-        instrumentator.instrument(
-            fastapi_app,
-            metric_subsystem=name,
-            latency_lowr_buckets=latency_buckets
-        )
-        instrumentator.expose(fastapi_app)
-
-        # Change the behaviour of the instrumentation middleware so that it
-        # returns fully interpolated paths in the prometheus labels instead of
-        # leaving them uninterpolated
-        middleware = [middleware for middleware in fastapi_app.user_middleware if "Prometheus" in middleware.cls.__name__]
-        middleware = middleware[0]
-        def _get_handler(self, request):
-            return request.url.path, True
-        middleware.cls._get_handler = _get_handler
-
-
-def _should_install_prometheus_middleware(fastapi_app):
-    # This avoids a bug on later version of fastapi where it crashes
-    # if you try to add a middleware more than once (useful in testing)
-    should_add_middleware = True
-    if fastapi_app.middleware_stack is not None:
-        should_add_middleware = False
-    return should_add_middleware
-
-
-def exception_response(status_code, exc):
+def exception_response(fastapi_app, status_code, exc):
     if (
         getattr(exc, "message", None)
         and "unhandled errors in a TaskGroup" in exc.message
     ):
         exc = exc.exceptions
-    return fa.responses.JSONResponse(
-        status_code=status_code, content={"detail": str(exc)}
+    trace_header_name = fastapi_app.state.chaiverse_server_config.trace_header_name
+    response = fa.responses.JSONResponse(
+        status_code=status_code,
+        content={"detail": str(exc)},
+        headers={trace_header_name: correlation_id.get() or ""},
     )
+    return response
 
 
 def _resolve_fastapi_app(fastapi_app):
