@@ -1,3 +1,3 @@
 from .firebase_database import _FirebaseDatabase, serialise_input
-from .redis_database import _RedisDatabase, _MultiRecordRedisDatabase
+from .redis_database import _AsyncRedisDatabase, _RedisDatabase, _MultiRecordRedisDatabase, _AsyncMultiRecordRedisDatabase
 from .mock_database import MockDatabase, _get_disallowed_firebase_keys
