@@ -2,13 +2,12 @@ from typing import Optional
 
 
 class _InfernoDatabaseAdapter():
-    def is_in_database(self, path: str):
-        value = self.get(path, shallow=True)
-        return value is not None
-
     def get(self, path: str, shallow: bool = True):
         raise NotImplementedError
 
+    def is_in_database(self, path: str):
+        raise NotImplementedError
+
     def set(self, path: str, value):
         raise NotImplementedError
 
