@@ -70,13 +70,13 @@ class PreferenceSummary(BaseModel):
         elo_rating = get_elo_rating(elo_score, elo_base_score, elo_base_rating) if elo_score is not None else None
         return elo_rating
 
-    def submission_metrics(self, submission_id: str, elo_base_id: str = None, elo_base_rating: float = None):
+    def submission_metrics(self, submission_id: str, elo_base_id: str = None, elo_base_rating: float = None, is_blend: bool = False):
         num_battles = self.submission_num_battles(submission_id)
         num_wins = self.submission_num_wins(submission_id)
         win_ratio = self.submission_win_ratio(submission_id)
 
         # blends do not have an elo rating
-        if elo_base_id is not None and num_battles >= ELO_REQUIRED_BATTLES:
+        if num_battles >= ELO_REQUIRED_BATTLES and not is_blend:
             celo_rating = self.submission_elo_rating(submission_id, elo_base_id=elo_base_id, elo_base_rating=elo_base_rating)
         else:
             celo_rating = None
@@ -84,9 +84,9 @@ class PreferenceSummary(BaseModel):
         metrics = dict(num_battles=num_battles, num_wins=num_wins, win_ratio=win_ratio, celo_rating=celo_rating)
         return metrics
 
-    def get_all_metrics_dict(self, elo_base_id: str = None, elo_base_rating: float = None):
+    def get_all_metrics_dict(self, elo_base_id: str = None, elo_base_rating: float = None, is_blend: bool = False):
         metrics = {
-            submission_id: self.submission_metrics(submission_id, elo_base_id=elo_base_id, elo_base_rating=elo_base_rating)
+            submission_id: self.submission_metrics(submission_id, elo_base_id=elo_base_id, elo_base_rating=elo_base_rating, is_blend=is_blend)
             for submission_id in self.submission_ids
         }
         return metrics
